@@ -47,11 +47,27 @@ class pkgRecords;
 enum { NO_PARSER, DEB_PARSER, STRIP_WS_PARSER, RPM_PARSER };
 
 // taken from apt (pkgcache.cc) to make our life easier 
-// (and added "RDepends" after "Obsoletes"
+// (and added "RDepends" as last element)
 static const char *DepTypeStr[] = 
-   {"",_("Depends"),_("PreDepends"),_("Suggests"),
-    _("Recommends"),_("Conflicts"),_("Replaces"),
-    _("Obsoletes"), _("Dependency of")};
+   {"",
+    _("Depends"),
+    _("PreDepends"),
+    _("Suggests"),
+    _("Recommends"),
+    _("Conflicts"),
+    _("Replaces"),
+    _("Obsoletes"), 
+    _("Breaks"),
+    _("Enhances"),
+    /* padding */
+    "", 
+    "",
+    "",
+    "",
+    "",
+    // make sure this is always the last member
+    _("Dependency of"),
+};
 
 typedef struct  {
    pkgCache::Dep::DepType type; // type as enum
@@ -66,15 +82,16 @@ typedef struct  {
 
 class RPackage {
 
-   protected:
-
+ public:
    RPackageLister *_lister;
 
+   protected:
+
+   string fullname;
    pkgRecords *_records;
    pkgDepCache *_depcache;
    pkgCache::PkgIterator *_package;
 
-
    // save the default candidate version to undo version selection
    string _defaultCandVer;
 
@@ -108,6 +125,10 @@ class RPackage {
       FPurge            = 1 << 17,
       FImportant        = 1 << 18,
       FOverrideVersion  = 1 << 19,
+      FIsAuto           = 1 << 20,
+      FIsGarbage        = 1 << 21,
+      FNowPolicyBroken  = 1 << 22,
+      FInstPolicyBroken  = 1 << 23,
    };
 
    enum UpdateImportance {
@@ -119,7 +140,7 @@ class RPackage {
 
    pkgCache::PkgIterator *package() { return _package; };
 
-   inline const char *name() { return _package->Name(); };
+   const char *name();
 
    const char *section();
    const char *priority();
@@ -128,22 +149,40 @@ class RPackage {
    const char *description();
    const char *installedFiles();
 
-   // get changelog file from the debian server (debian only of course)
+   string arch();
+
+   // package is also available for the native architecture
+   // (note that packages installed are never considered a duplicate
+   bool isMultiArchDuplicate();
+
+   // get changelog file from the debian server 
    string getChangelogFile(pkgAcquire *fetcher);
+   // get screenshot file from the debian server 
+   string getScreenshotFile(pkgAcquire *fetcher, bool thumb = true);
 
    vector<string> provides();
 
    // get all available versions (version, release)
    vector<pair<string, string> > getAvailableVersions();
 
-   // get origin of the package
-   string getCanidateOrigin();
+   // get origins url of the package (e.g. http://security.ubuntu.com)
+   vector<string> getCandidateOriginSiteUrls();
+   // get origin "archive" release header (e.g. karmic, karmic-updates)
+   vector<string> getCandidateOriginSuites();
+   // get origin "origin" release header (e.g. Ubuntu, 
+   string getCandidateOriginStr();
+
+   // get the release file for the givel origin label string
+   string getReleaseFileForOrigin(string label, string release);
 
    // get installed component (like main, contrib, non-free)
    string component();
    
    // get label of download site
    string label();
+
+   // get origin (Origin tag from the release file)
+   string origin();
    
    const char *maintainer();
    const char *vendor();
@@ -151,6 +190,12 @@ class RPackage {
    const char *installedVersion();
    long installedSize();
 
+   // get tag from pkg record
+   string findTagFromPkgRecord(const char *tag);
+
+   // get the raw package record
+   string getRawRecord(bool useCandidateVersion=true);
+
    // sourcepkg
    const char *srcPackage();
 
@@ -188,6 +233,9 @@ class RPackage {
       _boolFlags = flag ? (_boolFlags | FOrphaned) : (_boolFlags & ~FOrphaned);
    };
 
+   // set/unset the auto-installed flag
+   void setAuto(bool flag = true);
+
    void setNotify(bool flag = true);
 
    // Shallow doesnt remove things other pkgs depend on.
@@ -201,60 +249,12 @@ class RPackage {
    RPackage(RPackageLister *lister, pkgDepCache *depcache,
             pkgRecords *records, pkgCache::PkgIterator &pkg);
    ~RPackage();
+
+   private:
+   string getChangelogURI();
 };
 
 
-class RPackageStatus {
- public:
-   enum PkgStatus {
-      ToInstall, ToReInstall, ToUpgrade, ToDowngrade, ToRemove, ToPurge,
-      NotInstalled, NotInstalledLocked,
-      InstalledUpdated, InstalledOutdated, InstalledLocked,
-      IsBroken, IsNew,
-      N_STATUS_COUNT
-   };
-
- protected:
-   // the supported archive-labels and components
-   vector<string> supportedLabels;
-   vector<string> supportedComponents;
-   bool markUnsupported;
-
-   // this is the short string to load the icons
-   const char *PackageStatusShortString[N_STATUS_COUNT];
-   // this is the long string for the gui description of the state
-   const char *PackageStatusLongString[N_STATUS_COUNT];
-
-
-   // this does the actual work
-   int getStatus(RPackage *pkg);
-
-
- public:
-   RPackageStatus() : markUnsupported(false) {};
-   virtual ~RPackageStatus() {};
-
-   // this reads the pixmaps and the colors
-   virtual void init();
-
-   // here we get the description for the States
-   const char *getLongStatusString(int i) {
-      return PackageStatusLongString[i];
-   };
-   const char *getLongStatusString(RPackage *pkg) {
-      return PackageStatusLongString[getStatus(pkg)];
-   };
-
-   const char *getShortStatusString(int i) {
-      return PackageStatusShortString[i];
-   };
-
-   bool isSupported(RPackage *pkg);
-   
-};
-
-
-
 #endif
 
 // vim:ts=3:sw=3:et
