@@ -40,8 +40,7 @@
 
 using namespace std;
 
-// don't translate this, they are only used in the filter file
-char *RPatternPackageFilter::TypeName[] = {
+const char *RPatternPackageFilter::TypeName[] = {
    N_("Name"),
    N_("Description"),
    N_("Maintainer"),
@@ -54,6 +53,7 @@ char *RPatternPackageFilter::TypeName[] = {
    N_("Suggests"),
    N_("ReverseDepends"),
    N_("Origin"),
+   N_("Component"),
    NULL
 };
 
@@ -63,6 +63,7 @@ const char *RPFPattern = _("Pattern");
 const char *RPFSection = _("Section");
 const char *RPFPriority = _("Priority");
 const char *RPFReducedView = _("ReducedView");
+const char *RPFFile = _("File");
 
 int RSectionPackageFilter::count()
 {
@@ -84,7 +85,8 @@ string RSectionPackageFilter::section(int index)
 
 void RSectionPackageFilter::clear()
 {
-   _groups.erase(_groups.begin(), _groups.end());
+   //_groups.erase(_groups.begin(), _groups.end());
+   _groups.clear();
 }
 
 
@@ -219,7 +221,12 @@ bool RPatternPackageFilter::filterMaintainer(Pattern pat, RPackage *pkg)
 bool RPatternPackageFilter::filterDepends(Pattern pat, RPackage *pkg,
 					  pkgCache::Dep::DepType filterType)
 {
-   vector<RPackage::DepInformation> deps = pkg->enumDeps();
+   vector<DepInformation> deps = pkg->enumDeps();
+
+   if (pat.regexps.size() == 0) {
+      return true;
+   }
+   
    for(unsigned int i=0;i<deps.size();i++) {
       if(deps[i].type == filterType) {
 	    if (regexec(pat.regexps[0], deps[i].name, 0, NULL, 0) == 0) {
@@ -234,6 +241,11 @@ bool RPatternPackageFilter::filterProvides(Pattern pat, RPackage *pkg)
 {
    bool found = false;
    vector<string> provides = pkg->provides();
+
+   if (pat.regexps.size() == 0) {
+      return true;
+   }
+   
    for (unsigned int i = 0; i < provides.size(); i++) {
       if (regexec(pat.regexps[0], provides[i].c_str(), 0, NULL, 0) == 0) {
 	 found = true;
@@ -264,7 +276,12 @@ bool RPatternPackageFilter::filterWeakDepends(Pattern pat, RPackage *pkg)
 
 bool RPatternPackageFilter::filterRDepends(Pattern pat, RPackage *pkg)
 {
-   vector<RPackage::DepInformation> deps = pkg->enumRDeps();
+   vector<DepInformation> deps = pkg->enumRDeps();
+
+   if (pat.regexps.size() == 0) {
+      return true;
+   }
+   
    for(unsigned int i=0;i<deps.size();i++) {
       if (regexec(pat.regexps[0], deps[i].name, 0, NULL, 0) == 0) {
 	 return true;
@@ -273,10 +290,36 @@ bool RPatternPackageFilter::filterRDepends(Pattern pat, RPackage *pkg)
    return false;
 }
 bool RPatternPackageFilter::filterOrigin(Pattern pat, RPackage *pkg)
+{
+   bool found = false;
+   vector<string>origins = pkg->getCandidateOriginSiteUrls();
+
+   if (pat.regexps.size() == 0) {
+      return true;
+   }
+   
+   for (vector<string>::iterator it = origins.begin();
+        it != origins.end();
+        ++it)
+   {
+      if(regexec(pat.regexps[0],(*it).c_str(), 0, NULL, 0) == 0) {
+         found = true;
+      }
+   }
+
+   return found;
+}
+
+bool RPatternPackageFilter::filterComponent(Pattern pat, RPackage *pkg)
 {
    bool found = false;
    string origin;
-   origin = pkg->getCanidateOrigin();
+   origin = pkg->component();
+
+   if (pat.regexps.size() == 0) {
+      return true;
+   }
+   
    if(regexec(pat.regexps[0],origin.c_str(), 0, NULL, 0) == 0) {
       found = true;
    } 
@@ -287,10 +330,12 @@ bool RPatternPackageFilter::filterOrigin(Pattern pat, RPackage *pkg)
 bool RPatternPackageFilter::filter(RPackage *pkg)
 {
    bool found;
-   bool and_mode = _config->FindB("Synaptic::Filters::andMode", true);
+   //   bool and_mode = _config->FindB("Synaptic::Filters::andMode", true);
    bool globalfound = and_mode;
    bool useregexp = _config->FindB("Synaptic::UseRegexp", false);
 
+   bool debug = _config->FindB("Debug::Synaptic::Filters", "false");
+
    if (_patterns.size() == 0)
       return true;
 
@@ -335,10 +380,17 @@ bool RPatternPackageFilter::filter(RPackage *pkg)
       case Origin:
 	 found = filterOrigin(pat, pkg);
 	 break;
+      case Component:
+	 found = filterComponent(pat, pkg);
+	 break;
       default:
 	 cerr << "unknown pattern package filter (shouldn't happen) " << endl;
       }
 
+      if (found && debug)
+         clog << "RPatternPackageFilter::filter match for "
+              << pkg->name() << endl;
+
       // each filter is applied in AND fasion
       // that means a include depends "mono" and include name "sharp"
       // results in all packages that depends on "mono" AND have sharp in name
@@ -396,6 +448,8 @@ bool RPatternPackageFilter::write(ofstream &out, string pad)
    string pat;
    bool excl;
 
+   out << pad + "andMode " << and_mode << ";" << endl;
+
    out << pad + "patterns {" << endl;
 
    for (int i = 0; i < count(); i++) {
@@ -405,6 +459,7 @@ bool RPatternPackageFilter::write(ofstream &out, string pad)
    }
 
    out << pad + "};" << endl;
+
    return true;
 }
 
@@ -416,6 +471,8 @@ bool RPatternPackageFilter::read(Configuration &conf, string key)
    string pat;
    bool excl;
 
+   and_mode = conf.FindB(key + "::andMode", true);
+
    top = conf.Tree(string(key + "::patterns").c_str());
    if (top == NULL)
       return true;
@@ -450,6 +507,7 @@ RPatternPackageFilter::RPatternPackageFilter(RPatternPackageFilter &f)
       addPattern(f._patterns[i].where,
                  f._patterns[i].pattern, f._patterns[i].exclusive);
    }
+   f.and_mode = and_mode;
 }
 
 void RPatternPackageFilter::clear()
@@ -486,7 +544,9 @@ bool RStatusPackageFilter::filter(RPackage *pkg)
    }
 
    if (_status & MarkInstall) {
-      if (flags & RPackage::FInstall)
+      // this is a bit of a hack (to include reinstall here)
+      // it would be better to seperate this 
+      if ((flags & RPackage::FInstall) || (flags & RPackage::FReInstall))
          return true;
    }
 
@@ -562,6 +622,46 @@ bool RStatusPackageFilter::filter(RPackage *pkg)
          return true;
    }
 
+   if (_status & AutoInstalled) {
+      if (flags & RPackage::FIsAuto)
+         return true;
+   }
+
+   if (_status & Garbage) {
+      if (flags & RPackage::FIsGarbage)
+         return true;
+   }
+
+   if (_status & NowPolicyBroken) {
+      if (!(flags & RPackage::FInstalled))
+      {
+	 pkgCache::DepIterator D;
+	 bool inOr = false;
+	 // FIXME: or-dependencies are not considered properly
+	 for (D = pkg->package()->RevDependsList(); D.end() == false; D++)
+	 {	    
+	    if ((D->CompareOp & pkgCache::Dep::Or) == pkgCache::Dep::Or)
+	       inOr = true;
+	    else
+	       inOr = false;
+	    pkgCache::PkgIterator parent = D.ParentPkg();
+	    if(parent->CurrentVer != 0)
+	    {
+	       RPackage *p = pkg->_lister->getPackage(parent);
+	       if(p != NULL)
+		  if(p->getFlags() & RPackage::FNowPolicyBroken)
+		     return true;
+	    }
+	 }
+      }
+   }
+
+   if (_status & ManualInstalled) {
+      if ( !(flags & RPackage::FIsAuto) && 
+            (flags & RPackage::FInstalled))
+         return true;
+   }
+
    return false;
 }
 
@@ -638,7 +738,7 @@ void RReducedViewPackageFilter::addFile(string FileName)
 {
    FileFd F(FileName, FileFd::ReadOnly);
    if (_error->PendingError()) {
-      _error->Error(_("Internal Error: Could not open ReducedView file %s"),
+      _error->Error("Internal Error: Could not open ReducedView file %s",
                     FileName.c_str());
       return;
    }
@@ -694,6 +794,49 @@ bool RReducedViewPackageFilter::read(Configuration &conf, string key)
    return true;
 }
 
+bool RFilePackageFilter::addFile(string file)
+{
+  char str[255];
+  filename = file;
+  ifstream in(file.c_str());
+  if(!in) 
+     return false;
+  while(in) {
+     in.getline(str, 255);  
+     pkgs.insert(pkgs.begin(), string(str));
+  }
+  in.close();
+}
+
+bool RFilePackageFilter::filter(RPackage *pkg)
+{
+   if (pkgs.size() == 0)
+      return true;
+   return pkgs.find(pkg->name()) != pkgs.end();
+}
+
+bool RFilePackageFilter::write(ofstream &out, string pad)
+{
+   out << pad + "file {" << endl;
+   out << pad + "  \"" << filename << "\"; " << endl;
+   out << pad + "};" << endl;
+   return true;
+}
+
+bool RFilePackageFilter::read(Configuration &conf, string key)
+{
+   const Configuration::Item *top;
+
+   reset();
+
+   top = conf.Tree(string(key + "::file").c_str());
+   if (top != NULL) {
+      for (top = top->Child; top != NULL; top = top->Next)
+	 filename = top->Value;
+   }
+
+   return true;
+}
 
 bool RFilter::apply(RPackage *package)
 {
@@ -712,6 +855,9 @@ bool RFilter::apply(RPackage *package)
    if (!reducedview.filter(package))
       return false;
 
+   if (!file.filter(package))
+      return false;
+
    return true;
 }
 
@@ -807,4 +953,6 @@ bool RFilter::write(ofstream &out)
    return res;
 }
 
+
+
 // vim:sts=3:sw=3
