@@ -36,6 +36,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <time.h>
+#include <algorithm>
 
 #include "rpackagelister.h"
 #include "rpackagecache.h"
@@ -78,6 +79,9 @@ using namespace std;
 
 RPackageLister::RPackageLister()
    : _records(0), _progMeter(new OpProgress)
+#ifdef WITH_EPT
+   , _xapianDatabase(0)
+#endif
 {
    _cache = new RPackageCache();
 
@@ -85,16 +89,21 @@ RPackageLister::RPackageLister()
    _searchData.isRegex = false;
    _viewMode = _config->FindI("Synaptic::ViewMode", 0);
    _updating = true;
-   _sortMode = LIST_SORT_NAME;
+   _sortMode = LIST_SORT_DEFAULT;
 
    // keep order in sync with rpackageview.h 
-   _views.push_back(new RPackageViewSections(_packages));
-   _views.push_back(new RPackageViewStatus(_packages));
-   _filterView = new RPackageViewFilter(_packages);
+   _views.push_back(new RPackageViewSections(_nativeArchPackages));
+   _views.push_back(new RPackageViewStatus(_nativeArchPackages));
+   _views.push_back(new RPackageViewOrigin(_nativeArchPackages));
+   _filterView = new RPackageViewFilter(_nativeArchPackages);
    _views.push_back(_filterView);
-   _searchView =  new RPackageViewSearch(_packages);
-   _views.push_back(_searchView);   
-   //_views.push_back(new RPackageViewAlphabetic(_packages));
+   _searchView =  new RPackageViewSearch(_nativeArchPackages);
+   _views.push_back(_searchView);
+   // its import that we use "_packages" here instead of _nativeArchPackages
+   _views.push_back(new RPackageViewArchitecture(_packages));
+#ifdef WITH_EPT
+   openXapianIndex();
+#endif
 
    if (_viewMode >= _views.size())
       _viewMode = 0;
@@ -149,15 +158,20 @@ vector<string> RPackageLister::getSubViews()
 
 bool RPackageLister::setSubView(string newSubView)
 {
+   if(_config->FindB("Debug::Synaptic::View",false))
+      ioprintf(clog, "RPackageLister::setSubView(): newSubView '%s'\n", 
+	       newSubView.size() > 0 ? newSubView.c_str() : "(empty)");
+
    if(newSubView.empty())
       _selectedView->showAll();
    else
       _selectedView->setSelected(newSubView);
-   notifyPreChange(NULL);
 
-   reapplyFilter();
+   notifyChange(NULL);
 
-   notifyPostChange(NULL);
+   if(_config->FindB("Debug::Synaptic::View",false))
+      ioprintf(clog, "/RPackageLister::setSubView(): newSubView '%s'\n", 
+	       newSubView.size() > 0 ? newSubView.c_str() : "(empty)");
 
    return true;
 }
@@ -184,8 +198,11 @@ void RPackageLister::notifyPreChange(RPackage *pkg)
 
 void RPackageLister::notifyPostChange(RPackage *pkg)
 {
-   reapplyFilter();
+   if(_config->FindB("Debug::Synaptic::View",false))
+      ioprintf(clog, "RPackageLister::notifyPostChange(): '%s'\n",
+	       pkg == NULL ? "NULL" : pkg->name());
 
+   reapplyFilter();
 
    for (vector<RPackageObserver *>::const_iterator I =
         _packageObservers.begin(); I != _packageObservers.end(); I++) {
@@ -282,6 +299,9 @@ bool RPackageLister::openCache()
 {
    static bool firstRun = true;
 
+   if(_config->FindB("Debug::Synaptic::View",false))
+      clog << "RPackageLister::openCache()" << endl;
+
    // Flush old errors
    _error->Discard();
 
@@ -293,7 +313,7 @@ bool RPackageLister::openCache()
    if (!_cache->open(*_progMeter,lock)) {
       _progMeter->Done();
       _cacheValid = false;
-      return false;
+      return _error->Error("_cache->open() failed, please report.");
    }
    _progMeter->Done();
 
@@ -312,13 +332,10 @@ bool RPackageLister::openCache()
                              "Please report."), 2);
    }
 
-#ifdef HAVE_RPM
-   // be gentle and free memory
    if (_records)
       delete _records;
-#endif
-   
    _records = new pkgRecords(*deps);
+
    if (_error->PendingError()) {
       _cacheValid = false;
       return _error->Error(_("Internal error opening cache (%d). "
@@ -330,7 +347,11 @@ bool RPackageLister::openCache()
       delete (*I);
 
    int packageCount = deps->Head().PackageCount;
-   _packages.resize(packageCount);
+   _packages.clear();
+   _packages.reserve(packageCount);
+
+   _nativeArchPackages.clear();
+   _nativeArchPackages.reserve(packageCount);
 
    _packagesIndex.clear();
    _packagesIndex.resize(packageCount, -1);
@@ -338,6 +359,8 @@ bool RPackageLister::openCache()
    string pkgName;
    int count = 0;
 
+   bool showAllMultiArch = _config->FindB("Synaptic::ShowAllMultiArch", false);
+
    _installedCount = 0;
 
    map<string, RPackage *> pkgmap;
@@ -356,7 +379,12 @@ bool RPackageLister::openCache()
 
       RPackage *pkg = new RPackage(this, deps, _records, I);
       _packagesIndex[I->ID] = count;
-      _packages[count++] = pkg;
+      _packages.push_back(pkg);
+      count++;
+
+      // this is what is feed to the views
+      if (showAllMultiArch || !pkg->isMultiArchDuplicate())
+         _nativeArchPackages.push_back(pkg);
 
       pkgName = pkg->name();
 
@@ -379,9 +407,6 @@ bool RPackageLister::openCache()
       if (_roptions->getPackageLock(pkgName.c_str())) 
 	 pkg->setPinned(true);
 
-      for (unsigned int i = 0; i != _views.size(); i++)
-         _views[i]->addPackage(pkg);
-
       // Gather list of sections.
       if (I.Section())
          sectionSet.insert(I.Section());
@@ -391,8 +416,9 @@ bool RPackageLister::openCache()
 #endif
    }
 
-   // Truncate due to virtual packages which were skipped above.
-   _packages.resize(count);
+   // refresh the views
+   for (unsigned int i = 0; i != _views.size(); i++)
+      _views[i]->refresh();
 
    applyInitialSelection();
 
@@ -409,7 +435,47 @@ bool RPackageLister::openCache()
    return true;
 }
 
+#ifdef WITH_EPT
+bool RPackageLister::xapianIndexNeedsUpdate()
+{
+   struct stat buf;
+   
+   if(_config->FindB("Debug::Synaptic::Xapian",false))
+      std::cerr << "xapainIndexNeedsUpdate()" << std::endl;
 
+   // check the xapian index
+   if(FileExists("/usr/sbin/update-apt-xapian-index") && 
+      (!_xapianDatabase )) {
+      if(_config->FindB("Debug::Synaptic::Xapian",false))
+	 std::cerr << "xapain index not build yet" << std::endl;
+      return true;
+   } 
+
+   // compare timestamps, rebuild everytime, its now cheap(er) 
+   // because we use u-a-x-i --update
+   stat(_config->FindFile("Dir::Cache::pkgcache").c_str(), &buf);
+   if(ept::axi::timestamp() < buf.st_mtime) {
+      if(_config->FindB("Debug::Synaptic::Xapian",false))
+	 std::cerr << "xapian outdated " 
+		   << buf.st_mtime - ept::axi::timestamp()  << std::endl;
+      return true;
+   }
+
+   return false;
+}
+
+bool RPackageLister::openXapianIndex()
+{
+   if(_xapianDatabase)
+      delete _xapianDatabase;
+   try {
+      _xapianDatabase = new Xapian::Database(ept::axi::path_db());
+   } catch (Xapian::DatabaseOpeningError) {
+      return false;
+   };
+   return true;
+}
+#endif
 
 void RPackageLister::applyInitialSelection()
 {
@@ -506,42 +572,16 @@ bool RPackageLister::distUpgrade()
    return true;
 }
 
-static void qsSortByName(vector<RPackage *> &packages, int start, int end)
-{
-   int i, j;
-   RPackage *pivot, *tmp;
-
-   i = start;
-   j = end;
-   pivot = packages[(i + j) / 2];
-   do {
-      while (strcoll(packages[i]->name(), pivot->name()) < 0)
-         i++;
-      while (strcoll(pivot->name(), packages[j]->name()) < 0)
-         j--;
-      if (i <= j) {
-         tmp = packages[i];
-         packages[i] = packages[j];
-         packages[j] = tmp;
-         i++;
-         j--;
-      }
-   } while (i <= j);
-
-   if (start < j)
-      qsSortByName(packages, start, j);
-   if (i < end)
-      qsSortByName(packages, i, end);
-}
-
 void RPackageLister::reapplyFilter()
 {
    // PORTME
    if (_updating)
       return;
 
-   //   cout << "RPackageLister::reapplyFilter()" << endl;
+   if(_config->FindB("Debug::Synaptic::View",false))
+      clog << "RPackageLister::reapplyFilter()" << endl;
 
+   _selectedView->refresh();
    _viewPackages.clear();
    _viewPackagesIndex.clear();
    _viewPackagesIndex.resize(_packagesIndex.size(), -1);
@@ -554,6 +594,7 @@ void RPackageLister::reapplyFilter()
       }
    }
 
+
    sortPackages(_sortMode);
 }
 
@@ -583,7 +624,7 @@ struct componentSortFunc {
 
 struct sectionSortFunc {
    bool operator() (RPackage *x, RPackage *y) {
-      return x->section() < y->section();
+      return std::strcmp(x->section(), y->section())<0;
 }};
 
 // version string compare
@@ -611,21 +652,23 @@ struct instVersionSortFunc {
 		       x->installedVersion());
 }};
 
-struct supportedSortFunc {
+struct supportedPartFunc {
  protected:
    bool _ascent;
    RPackageStatus _status;
  public:
-   supportedSortFunc(bool ascent, RPackageStatus &s) 
+   supportedPartFunc(bool ascent, RPackageStatus &s) 
       : _ascent(ascent), _status(s) {};
-   bool operator() (RPackage *x, RPackage *y) {
-      if(_ascent)
-	 return _status.isSupported(y) < _status.isSupported(x);
-      else
-	 return _status.isSupported(y) > _status.isSupported(x);
+   bool operator() (RPackage *x) {
+      return (_ascent==_status.isSupported(x));
    }
 };
 
+struct nameSortFunc {
+   bool operator() (RPackage *x, RPackage *y) {
+      return std::strcmp(x->name(), y->name())<0;
+}};
+
 
 
 void RPackageLister::sortPackages(vector<RPackage *> &packages, 
@@ -635,9 +678,22 @@ void RPackageLister::sortPackages(vector<RPackage *> &packages,
    if (packages.empty())
       return;
 
+   if(_config->FindB("Debug::Synaptic::View",false))
+      clog << "RPackageLister::sortPackages(): " << packages.size() << endl;
+
+   /* Always sort by name to have packages ordered inside another sort 
+    * criteria */
+   sort(packages.begin(), packages.end(), 
+      sortFunc<nameSortFunc>(true));
+
    switch(mode) {
-   case LIST_SORT_NAME:
-      qsSortByName(packages, 0, packages.size() - 1);
+   case LIST_SORT_NAME_ASC:
+   case LIST_SORT_DEFAULT:
+      // Do nothing, already done
+      break;
+   case LIST_SORT_NAME_DES:
+      sort(packages.begin(), packages.end(), 
+		  sortFunc<nameSortFunc>(false));
       break;
    case LIST_SORT_SIZE_ASC:
       stable_sort(packages.begin(), packages.end(), 
@@ -656,44 +712,36 @@ void RPackageLister::sortPackages(vector<RPackage *> &packages,
 		  sortFunc<dlSizeSortFunc>(false));
       break;
    case LIST_SORT_COMPONENT_ASC:
-      qsSortByName(packages, 0, packages.size() - 1);
       stable_sort(packages.begin(), packages.end(), 
 		  sortFunc<componentSortFunc>(true));
       break;
    case LIST_SORT_COMPONENT_DES:
-      qsSortByName(packages, 0, packages.size() - 1);
-      stable_sort(packages.begin(), packages.end(), 		  
+      stable_sort(packages.begin(), packages.end(), 
 		  sortFunc<componentSortFunc>(false));
       break;
    case LIST_SORT_SECTION_ASC:
-      qsSortByName(packages, 0, packages.size() - 1);
       stable_sort(packages.begin(), packages.end(), 
 		  sortFunc<sectionSortFunc>(true));
       break;
    case LIST_SORT_SECTION_DES:
-      qsSortByName(packages, 0, packages.size() - 1);
       stable_sort(packages.begin(), packages.end(), 
 		  sortFunc<sectionSortFunc>(false));
       break;
    case LIST_SORT_STATUS_ASC:
-      qsSortByName(packages, 0, packages.size() - 1);
       stable_sort(packages.begin(), packages.end(), 
 		  sortFunc<statusSortFunc>(true));
       break;
    case LIST_SORT_STATUS_DES:
-      qsSortByName(packages, 0, packages.size() - 1);
       stable_sort(packages.begin(), packages.end(), 
 		  sortFunc<statusSortFunc>(false));
       break;
    case LIST_SORT_SUPPORTED_ASC:
-      qsSortByName(packages, 0, packages.size() - 1);
-      stable_sort(packages.begin(), packages.end(), 
-		  supportedSortFunc(true, _pkgStatus));
+      stable_partition(packages.begin(), packages.end(), 
+		  supportedPartFunc(false, _pkgStatus));
       break;
    case LIST_SORT_SUPPORTED_DES:
-      qsSortByName(packages, 0, packages.size() - 1);
-      stable_sort(packages.begin(), packages.end(), 
-		  supportedSortFunc(false, _pkgStatus));
+      stable_partition(packages.begin(), packages.end(), 
+		  supportedPartFunc(true, _pkgStatus));
       break;
    case LIST_SORT_VERSION_ASC:
       stable_sort(packages.begin(), packages.end(), 
@@ -952,7 +1000,7 @@ bool RPackageLister::getStateChanges(RPackageLister::pkgState &state,
                                      vector<RPackage *> &toRemove,
                                      vector<RPackage *> &toDowngrade,
 				     vector<RPackage *> &notAuthenticated,
-                                     vector<RPackage *> &exclude,
+                                     const vector<RPackage *> &exclude,
                                      bool sorted)
 {
    bool changed = false;
@@ -1023,6 +1071,7 @@ void RPackageLister::saveState(RPackageLister::pkgState &state)
 void RPackageLister::restoreState(RPackageLister::pkgState &state)
 {
    pkgDepCache *deps = _cache->deps();
+   pkgDepCache::ActionGroup group(*deps);
 
    for (unsigned i = 0; i < _packages.size(); i++) {
       RPackage *pkg = _packages[i];
@@ -1039,7 +1088,9 @@ void RPackageLister::restoreState(RPackageLister::pkgState &state)
             deps->MarkDelete(*(pkg->package()), oldflags & RPackage::FPurge);
          } else if (oldflags & RPackage::FKeep) {
             deps->MarkKeep(*(pkg->package()), false);
-         }
+	 }
+	 // fix the auto flag
+	 deps->MarkAuto(*pkg->package(), (oldflags & RPackage::FIsAuto));
       }
    }
    notifyChange(NULL);
@@ -1054,7 +1105,7 @@ bool RPackageLister::getStateChanges(RPackageLister::pkgState &state,
                                      vector<RPackage *> &toRemove,
                                      vector<RPackage *> &toDowngrade,
 				     vector<RPackage *> &notAuthenticated,
-                                     vector<RPackage *> &exclude,
+                                     const vector<RPackage *> &exclude,
                                      bool sorted)
 {
    bool changed = false;
@@ -1238,6 +1289,7 @@ bool RPackageLister::updateCache(pkgAcquireStatus *status, string &error)
    // Get the source list
    //pkgSourceList List;
    _cache->list()->ReadMainList();
+
    // Lock the list directory
    FileFd Lock;
    if (_config->FindB("Debug::NoLocking", false) == false) {
@@ -1249,12 +1301,26 @@ bool RPackageLister::updateCache(pkgAcquireStatus *status, string &error)
 
    _updating = true;
 
+
+#ifndef HAVE_RPM
+// apt-0.7.10 has the new UpdateList code in algorithms, we use it
+   string s;
+   bool res = ListUpdate(*status, *_cache->list(), 5000);
+   if(res == false)
+   {
+      while(!_error->empty())
+      {
+	 bool isError = _error->PopMessage(s);
+	 error += s;
+      }
+   }
+   return res;
+#else
    // Create the download object
    pkgAcquire Fetcher(status);
 
    bool Failed = false;
 
-#if HAVE_RPM
    if (_cache->list()->GetReleases(&Fetcher) == false)
       return false;
    Fetcher.Run();
@@ -1269,21 +1335,13 @@ bool RPackageLister::updateCache(pkgAcquireStatus *status, string &error)
       _error->Warning(_("Release files for some repositories could not be "
                         "retrieved or authenticated. Such repositories are "
                         "being ignored."));
-#endif /* HAVE_RPM */
 
    if (!_cache->list()->GetIndexes(&Fetcher))
       return false;
 
-// apt-rpm does not support the pulseInterval
-#ifdef HAVE_RPM 
    // Run it
    if (Fetcher.Run() == pkgAcquire::Failed)
       return false;
-#else
-   if (Fetcher.Run(50000) == pkgAcquire::Failed)
-      return false;
-#endif
-
 
    //bool AuthFailed = false;
    Failed = false;
@@ -1316,6 +1374,7 @@ bool RPackageLister::updateCache(pkgAcquireStatus *status, string &error)
       return false; 
    }
    return true;
+#endif
 }
 
 bool RPackageLister::getDownloadUris(vector<string> &uris)
@@ -1360,11 +1419,10 @@ bool RPackageLister::commitChanges(pkgAcquireStatus *status,
          warning(_("Ignoring invalid record(s) in sources.list file!"));
    }
 
-   pkgPackageManager *PM;
-   PM = _system->CreatePM(_cache->deps());
-   RPackageManager *rPM = new RPackageManager(PM);
+   pkgPackageManager *rPM;
+   rPM = _system->CreatePM(_cache->deps());
 
-   if (!PM->GetArchives(&fetcher, _cache->list(), _records) ||
+   if (!rPM->GetArchives(&fetcher, _cache->list(), _records) ||
        _error->PendingError())
       goto gave_wood;
 
@@ -1414,7 +1472,7 @@ bool RPackageLister::commitChanges(pkgAcquireStatus *status,
 
          serverError = getServerErrorMessage(errm);
 
-         _error->Warning(tmp.str().c_str());
+         _error->Warning("%s", tmp.str().c_str());
          Failed = true;
       }
 
@@ -1442,7 +1500,7 @@ bool RPackageLister::commitChanges(pkgAcquireStatus *status,
             goto gave_wood;
       }
       // Try to deal with missing package files
-      if (Failed == true && PM->FixMissing() == false) {
+      if (Failed == true && rPM->FixMissing() == false) {
          _error->Error(_("Unable to correct missing packages"));
          goto gave_wood;
       }
@@ -1478,7 +1536,7 @@ bool RPackageLister::commitChanges(pkgAcquireStatus *status,
       // Reload the fetcher object and loop again for media swapping
       fetcher.Shutdown();
 
-      if (!PM->GetArchives(&fetcher, _cache->list(), _records))
+      if (!rPM->GetArchives(&fetcher, _cache->list(), _records))
          goto gave_wood;
    }
 
@@ -1491,12 +1549,10 @@ bool RPackageLister::commitChanges(pkgAcquireStatus *status,
    if(_config->FindB("Synaptic::Log::Changes",true))
       writeCommitLog();
 
-   delete PM;
    delete rPM;
    return Ret;
 
  gave_wood:
-   delete PM;
    delete rPM;
    return false;
 }
@@ -1702,6 +1758,10 @@ bool RPackageLister::cleanPackageCache(bool forceClean)
    return true;
 }
 
+void RPackageLister::refreshView()
+{
+   _selectedView->refresh();
+}
 
 bool RPackageLister::writeSelections(ostream &out, bool fullState)
 {
@@ -1729,8 +1789,10 @@ bool RPackageLister::readSelections(istream &in)
       ACTION_UNINSTALL
    };
    map<string, int> actionMap;
+   pkgDepCache::ActionGroup group(*_cache->deps());
 
    while (in.eof() == false) {
+
       in.getline(Buffer, sizeof(Buffer));
       CurLine++;
 
@@ -1780,7 +1842,9 @@ bool RPackageLister::readSelections(istream &in)
 	    Fix.Protect(Pkg);
             switch ((*I).second) {
                case ACTION_INSTALL:
-		  if(_config->FindB("Volatile::SetSelectionsNoFix","false"))
+		  if(_config->FindB("Volatile::SetSelectionDoReInstall",false))
+		     Cache.SetReInstall(Pkg, true);
+		  if(_config->FindB("Volatile::SetSelectionsNoFix",false))
 		     Cache.MarkInstall(Pkg, false);
 		  else
 		     Cache.MarkInstall(Pkg, true);
@@ -1803,6 +1867,11 @@ bool RPackageLister::readSelections(istream &in)
       _progMeter->Done();
       Fix.InstallProtect();
       Fix.Resolve(true);
+
+      // refresh all views
+      for (unsigned int i = 0; i != _views.size(); i++)
+         _views[i]->refresh();
+
    }
 
    return true;
@@ -1810,6 +1879,7 @@ bool RPackageLister::readSelections(istream &in)
 
 bool RPackageLister::addArchiveToCache(string archive, string &pkgname)
 {
+#ifndef HAVE_RPM
    //cout << "addArchiveToCache() " << archive << endl;
 
    // do sanity checking on the file (do we need this 
@@ -1842,7 +1912,9 @@ bool RPackageLister::addArchiveToCache(string archive, string &pkgname)
    
    // correct version?
    string debVer = tag.FindS("Version");
-   string candVer = pkg->availableVersion();
+   string candVer = "_invalid_";
+   if(pkg->availableVersion() != NULL)
+      candVer = pkg->availableVersion();
    if(debVer != candVer) {
       cerr << "Ignoring " << pkgname << " (different versions: "
 	   << debVer << " != " << candVer  << endl;
@@ -1872,6 +1944,149 @@ bool RPackageLister::addArchiveToCache(string archive, string &pkgname)
    CopyFile(in, out);
 
    return true;
+#else
+   return false;
+#endif
+}
+
+
+#ifdef WITH_EPT
+bool RPackageLister::limitBySearch(string searchString)
+{
+   //cerr << "limitBySearch(): " << searchString << endl;
+    if (ept::axi::timestamp() == 0)
+        return false;
+   return xapianSearch(searchString);
+}
+
+bool RPackageLister::xapianSearch(string unsplitSearchString)
+{
+   //std::cerr << "RPackageLister::xapianSearch()" << std::endl;
+   static const int defaultQualityCutoff = 15;
+   int qualityCutoff = _config->FindI("Synaptic::Xapian::qualityCutoff", 
+                                      defaultQualityCutoff);
+    if (ept::axi::timestamp() == 0) 
+        return false;
+
+   try {
+      int maxItems = _xapianDatabase->get_doccount();
+      Xapian::Enquire enquire(*_xapianDatabase);
+      Xapian::QueryParser parser;
+      parser.set_database(*_xapianDatabase);
+      parser.add_prefix("name","XP");
+      parser.add_prefix("section","XS");
+      // default op is AND to narrow down the resultset
+      parser.set_default_op( Xapian::Query::OP_AND );
+   
+      /* Workaround to allow searching an hyphenated package name using a prefix (name:)
+       * LP: #282995
+       * Xapian currently doesn't support wildcard for boolean prefix and 
+       * doesn't handle implicit wildcards at the end of hypenated phrases.
+       *
+       * e.g searching for name:ubuntu-res will be equivalent to 'name:ubuntu res*'
+       * however 'name:(ubuntu* res*) won't return any result because the 
+       * index is built with the full package name
+       */
+      // Always search for the package name
+      string xpString = "name:";
+      string::size_type pos = unsplitSearchString.find_first_of(" ,;");
+      if (pos > 0) {
+          xpString += unsplitSearchString.substr(0,pos);
+      } else {
+          xpString += unsplitSearchString;
+      }
+      Xapian::Query xpQuery = parser.parse_query(xpString);
+
+      pos = 0;
+      while ( (pos = unsplitSearchString.find("-", pos)) != string::npos ) {
+         unsplitSearchString.replace(pos, 1, " ");
+         pos+=1;
+      }
+
+      if(_config->FindB("Debug::Synaptic::Xapian",false)) 
+         std::cerr << "searching for : " << unsplitSearchString << std::endl;
+      
+      // Build the query
+      // apply a weight factor to XP term to increase relevancy on package name
+      Xapian::Query query = parser.parse_query(unsplitSearchString, 
+         Xapian::QueryParser::FLAG_WILDCARD |
+         Xapian::QueryParser::FLAG_BOOLEAN |
+         Xapian::QueryParser::FLAG_PARTIAL);
+      query = Xapian::Query(Xapian::Query::OP_OR, query, 
+              Xapian::Query(Xapian::Query::OP_SCALE_WEIGHT, xpQuery, 3));
+      enquire.set_query(query);
+      Xapian::MSet matches = enquire.get_mset(0, maxItems);
+
+      if(_config->FindB("Debug::Synaptic::Xapian",false)) {
+         cerr << "enquire: " << enquire.get_description() << endl;
+         cerr << "matches estimated: " << matches.get_matches_estimated() << " results found" << endl;
+      }
+
+      // Retrieve the results
+      int top_percent = 0;
+      _viewPackages.clear();
+      for (Xapian::MSetIterator i = matches.begin(); i != matches.end(); ++i)
+      {
+         RPackage* pkg = getPackage(i.get_document().get_data());
+         // Filter out results that apt doesn't know
+         if (!pkg || !_selectedView->hasPackage(pkg))
+            continue;
+
+         // Save the confidence interval of the top value, to use it as
+         // a reference to compute an adaptive quality cutoff
+         if (top_percent == 0)
+            top_percent = i.get_percent();
+   
+         // Stop producing if the quality goes below a cutoff point
+         if (i.get_percent() < qualityCutoff * top_percent / 100)
+         {
+            cerr << "Discarding: " << i.get_percent() << " over " << qualityCutoff * top_percent / 100 << endl;
+            break;
+         }
+   
+         if(_config->FindB("Debug::Synaptic::Xapian",false)) 
+            cerr << i.get_rank() + 1 << ": " << i.get_percent() << "% docid=" << *i << "	[" << i.get_document().get_data() << "]" << endl;
+         _viewPackages.push_back(pkg);
+         }
+      // re-apply sort criteria only if an explicit search is set
+      if (_sortMode != LIST_SORT_DEFAULT)
+          sortPackages(_sortMode);
+      return true;
+   } catch (const Xapian::Error & error) {
+      /* We are here if a Xapian call failed. The main cause is a parser exception.
+       * The error message is always in English currently. 
+       * The possible parser errors are:
+       *    Unknown range operation
+       *    parse error
+       *    Syntax: <expression> AND <expression>
+       *    Syntax: <expression> AND NOT <expression>
+       *    Syntax: <expression> NOT <expression>
+       *    Syntax: <expression> OR <expression>
+       *    Syntax: <expression> XOR <expression>
+       */
+      cerr << "Exception in RPackageLister::xapianSearch():" << error.get_msg() << endl;
+      return false;
+   }
+}
+#else
+bool RPackageLister::limitBySearch(string searchString)
+{
+   return false;
+}
+
+bool RPackageLister::xapianSearch(string searchString) 
+{ 
+   return false; 
+}
+#endif
+
+bool RPackageLister::isMultiarchSystem()
+{
+#ifdef WITH_APT_MULTIARCH_SUPPORT
+   return (APT::Configuration::getArchitectures().size() > 1);
+#else
+   return false;
+#endif
 }
 
 // vim:ts=3:sw=3:et
