@@ -23,7 +23,8 @@
 #include <apt-pkg/strutl.h>
 #include <cassert>
 #include "gtkpkglist.h"
-#include "rgmisc.h"
+#include "rgutils.h"
+#include "rgpackagestatus.h"
 #include "rpackagelister.h"
 
 using namespace std;
@@ -227,13 +228,17 @@ static void gtk_pkg_list_init(GtkPkgList *pkg_list)
    //cout << "list_init()" << endl;
    pkg_list->n_columns = N_COLUMNS;
    pkg_list->column_headers[0] = GDK_TYPE_PIXBUF;
-   pkg_list->column_headers[1] = G_TYPE_STRING;
+   pkg_list->column_headers[1] = GDK_TYPE_PIXBUF;
    pkg_list->column_headers[2] = G_TYPE_STRING;
    pkg_list->column_headers[3] = G_TYPE_STRING;
    pkg_list->column_headers[4] = G_TYPE_STRING;
    pkg_list->column_headers[5] = G_TYPE_STRING;
-   pkg_list->column_headers[6] = GDK_TYPE_COLOR;
-   pkg_list->column_headers[7] = G_TYPE_POINTER;
+   pkg_list->column_headers[6] = G_TYPE_STRING;
+   pkg_list->column_headers[7] = G_TYPE_STRING;
+   pkg_list->column_headers[8] = G_TYPE_STRING;
+   pkg_list->column_headers[9] = G_TYPE_STRING;
+   pkg_list->column_headers[10] = GDK_TYPE_COLOR;
+   pkg_list->column_headers[11] = G_TYPE_POINTER;
 }
 
 /**
@@ -380,6 +385,7 @@ gtk_pkg_list_get_value(GtkTreeModel *tree_model,
    g_value_init(value, GTK_PKG_LIST(tree_model)->column_headers[column]);
 
    RPackage *pkg = (RPackage *) iter->user_data;
+
 #ifdef DEBUG_LIST_FULL
    if (column == NAME_COLUMN) {
       cout << "get_value: column: " << column;
@@ -405,40 +411,42 @@ gtk_pkg_list_get_value(GtkTreeModel *tree_model,
          g_value_set_string(value, str);
          break;
       case PKG_SIZE_COLUMN:
-         if (pkg == NULL)
-            return;
          if (pkg->installedVersion()) {
             str = SizeToStr(pkg->installedSize()).c_str();
             g_value_set_string(value, str);
          }
          break;
+      case PKG_DOWNLOAD_SIZE_COLUMN:
+	 str = SizeToStr(pkg->availablePackageSize()).c_str();
+	 g_value_set_string(value, str);
+         break;
+      case SECTION_COLUMN:
+	 str = pkg->section();
+	 if(str != NULL)
+	    g_value_set_string(value, str);
+	 break;
+      case COMPONENT_COLUMN:
+	 str = pkg->component().c_str();
+	 if(str)
+	    g_value_set_string(value, str);
+	 break;
       case INSTALLED_VERSION_COLUMN:
-         if (pkg == NULL)
-            return;
          str = pkg->installedVersion();
          g_value_set_string(value, str);
          break;
       case AVAILABLE_VERSION_COLUMN:
-         if (pkg == NULL)
-            return;
          str = pkg->availableVersion();
          g_value_set_string(value, str);
          break;
       case DESCR_COLUMN:
-         if (pkg == NULL)
-            return;
          str = utf8(pkg->summary());
          g_value_set_string(value, str);
          break;
       case PKG_COLUMN:
-         if (pkg == NULL)
-            return;
          g_value_set_pointer(value, pkg);
          break;
       case COLOR_COLUMN:
        {
-          if (pkg == NULL)
-             return;
 	  if(_config->FindB("Synaptic::UseStatusColors", TRUE) == FALSE) 
 	     return;
           GdkColor *bg;
@@ -446,11 +454,18 @@ gtk_pkg_list_get_value(GtkTreeModel *tree_model,
           g_value_set_boxed(value, bg);
           break;
        }
-      case PIXMAP_COLUMN:
+      case SUPPORTED_COLUMN:
        {
           if (pkg == NULL)
              return;
           GdkPixbuf *pix;
+          pix = RGPackageStatus::pkgStatus.getSupportedPix(pkg);
+          g_value_set_object(value, pix);
+          break;
+       }
+      case PIXMAP_COLUMN:
+       {
+          GdkPixbuf *pix;
           pix = RGPackageStatus::pkgStatus.getPixbuf(pkg);
           g_value_set_object(value, pix);
           break;
@@ -484,6 +499,7 @@ gtk_pkg_list_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
    cout << "iter_next()  " << endl;
    cout << "old: " << oldpkg->name() << " [" << old << "] " << endl;
    cout << "new: " << pkg->name() << " [" << i << "] " << endl;
+   cout << "viewPackagesSize: " << pkg_list->_lister->viewPackagesSize() << endl;
 #endif
 
    iter->stamp = 140677;
@@ -509,6 +525,10 @@ gtk_pkg_list_iter_children(GtkTreeModel *tree_model,
 
    GtkPkgList *pkg_list = (GtkPkgList *) tree_model;
 
+   // should never happen, but does apparently with atk turned on
+   if (pkg_list->_lister->getViewPackages().size() == 0)
+      return FALSE;
+
    RPackage *pkg = pkg_list->_lister->getViewPackage(0);
 
    iter->stamp = 140677;
@@ -634,14 +654,41 @@ static void gtk_pkg_list_sort(GtkPkgList *pkg_list)
       else
 	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_STATUS_DES);
       break;
+   case SUPPORTED_COLUMN:
+      if(pkg_list->order)
+	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_SUPPORTED_ASC);
+      else
+	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_SUPPORTED_DES);
+      break;
    case NAME_COLUMN:
-      pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_NAME);
+      if(pkg_list->order)
+     pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_NAME_DES);
+      else
+     pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_NAME_ASC);
       break;
    case PKG_SIZE_COLUMN:
       if(pkg_list->order)
-	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_SIZE_ASC);
-      else
 	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_SIZE_DES);
+      else
+	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_SIZE_ASC);
+      break;
+   case COMPONENT_COLUMN:
+      if(pkg_list->order)
+	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_COMPONENT_DES);
+      else
+	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_COMPONENT_ASC);
+      break;
+   case SECTION_COLUMN:
+      if(pkg_list->order)
+	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_SECTION_DES);
+      else
+	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_SECTION_ASC);
+      break;
+   case PKG_DOWNLOAD_SIZE_COLUMN:
+      if(pkg_list->order)
+	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_DLSIZE_DES);
+      else
+	 pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_DLSIZE_ASC);
       break;
    case AVAILABLE_VERSION_COLUMN:
       if(pkg_list->order)
@@ -657,7 +704,7 @@ static void gtk_pkg_list_sort(GtkPkgList *pkg_list)
       break;
    default:
       //cerr << "unknown sort column: " << pkg_list->sort_column_id << endl;
-      pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_NAME);
+      pkg_list->_lister->sortPackages(RPackageLister::LIST_SORT_DEFAULT);
    }
 }
 
