@@ -20,10 +20,10 @@
  * USA
  */
 
-#include "rggladewindow.h"
+#include "rggtkbuilderwindow.h"
 #include "rgmainwindow.h"
 
-class RGPreferencesWindow:public RGGladeWindow {
+class RGPreferencesWindow:public RGGtkBuilderWindow {
    bool _blockAction;
 
    enum {TREE_CHECKBOX_COLUMN, TREE_VISIBLE_NAME_COLUMN, TREE_NAME_COLUMN};
@@ -39,6 +39,10 @@ class RGPreferencesWindow:public RGGladeWindow {
    static const char *column_visible_names[];
    static const gboolean column_visible_defaults[];
 
+   static const char *removal_actions[];
+   static const char *update_ask[];
+   static const char *upgrade_method[];
+
    RGMainWindow *_mainWin;
    RPackageLister *_lister;
    // option buttons
@@ -55,21 +59,32 @@ class RGPreferencesWindow:public RGGladeWindow {
    GtkWidget *_cacheClean;
    GtkWidget *_cacheAutoClean;
 
+   GtkWidget *_delHistory;
+   GtkWidget *_keepHistory;
+   GtkWidget *_spinDelHistory;
+
    GtkWidget *_pathT;
    GtkWidget *_sizeT;
    GtkWidget *_maxUndoE;
-   GtkWidget *_optionmenuDel;
+   GtkWidget *_comboRemovalAction;
+   GtkWidget *_comboUpdateAsk;
+   GtkWidget *_comboUpgradeMethod;
    GtkWidget *_useProxy;
 
    // policy settings
-   GtkWidget *_optionmenuDefaultDistro;
+   GtkWidget *_comboDefaultDistro;
    string _defaultDistro;
 
+   bool _dirty;
+
    int columnPos[6];
 
    // distro selection
-   static void onArchiveSelection(GtkWidget *self, void *data);
+   static void cbArchiveSelection(GtkWidget *self, void *data);
+   static void cbRadioDistributionChanged(GtkWidget *self, void *data);
    bool distroChanged;
+   // the http proxy configuration
+   static void cbHttpProxyEntryChanged(GtkWidget *self, void *data);
 
    // treeview stuff
    void readTreeViewValues();
@@ -89,16 +104,16 @@ class RGPreferencesWindow:public RGGladeWindow {
    void saveGeneral();
    void saveColumnsAndFonts();
    void saveColors();
-   void saveTempFiles();
+   void saveFiles();
    void saveNetwork();
-   void saveExpert();
+   void saveDistribution();
 
    void readGeneral();
    void readColumnsAndFonts();
    void readColors();
-   void readTempFiles();
+   void readFiles();
    void readNetwork();
-   void readExpert();
+   void readDistribution();
 
 
    static void closeAction(GtkWidget *self, void *data);
@@ -106,6 +121,7 @@ class RGPreferencesWindow:public RGGladeWindow {
    static void clearCacheAction(GtkWidget *self, void *data);
 
    static void colorClicked(GtkWidget *self, void *data);
+   static void buttonAuthenticationClicked(GtkWidget *self, void *data);
 
    static void useProxyToggled(GtkWidget *self, void *data);
 
