@@ -26,9 +26,10 @@
 #include <apt-pkg/fileutl.h>
 
 #include <assert.h>
+#include <string>
 #include "i18n.h"
 #include "rguserdialog.h"
-#include "rgmisc.h"
+#include "rgutils.h"
 
 static void actionResponse(GtkDialog *dialog, gint id, gpointer user_data)
 {
@@ -39,7 +40,7 @@ static void actionResponse(GtkDialog *dialog, gint id, gpointer user_data)
 bool RGUserDialog::showErrors()
 {
    GtkWidget *dia;
-   string err,warn;
+   std::string err,warn;
    
    if (_error->empty())
       return false;
@@ -72,7 +73,7 @@ bool RGUserDialog::showErrors()
 				NULL);
    gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG(dia),
 				  g_strdup_printf("<b><big>%s</big></b>\n\n%s",
-				                  _("An error occured"), 
+				                  _("An error occurred"), 
 				                  _("The following details "
 						    "are provided:")));
    gtk_widget_set_size_request(dia, 500, 300);
@@ -85,6 +86,7 @@ bool RGUserDialog::showErrors()
    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textview), GTK_WRAP_WORD);
    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(textview), 3);
    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(textview), FALSE);
+   gtk_text_view_set_editable(GTK_TEXT_VIEW(textview), FALSE);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll), 
                                        GTK_SHADOW_IN);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
@@ -99,8 +101,10 @@ bool RGUserDialog::showErrors()
    int id = _config->FindI("Volatile::ParentWindowId", -1);
    if (id > 0) {
       GdkWindow *win = gdk_window_foreign_new(id);
-      gtk_widget_realize(dia);
-      gdk_window_set_transient_for(GDK_WINDOW(dia->window), win);
+      if(win) {
+	 gtk_widget_realize(dia);
+	 gdk_window_set_transient_for(GDK_WINDOW(dia->window), win);
+      }
    }
 
    gtk_dialog_run(GTK_DIALOG(dia));
@@ -188,8 +192,10 @@ bool RGUserDialog::message(const char *msg,
    int id = _config->FindI("Volatile::ParentWindowId", -1);
    if (id > 0) {
       GdkWindow *win = gdk_window_foreign_new(id);
-      gtk_widget_realize(dia);
-      gdk_window_set_transient_for(GDK_WINDOW(dia->window), win);
+      if(win) {
+	 gtk_widget_realize(dia);
+	 gdk_window_set_transient_for(GDK_WINDOW(dia->window), win);
+      }
    }
 
    gtk_dialog_run(GTK_DIALOG(dia));
@@ -197,31 +203,39 @@ bool RGUserDialog::message(const char *msg,
    return (res == GTK_RESPONSE_OK) || (res == GTK_RESPONSE_YES) || (res == GTK_RESPONSE_CLOSE);
 }
 
-// RGGladeUserDialog
-RGGladeUserDialog::RGGladeUserDialog(RGWindow *parent, const char *name)
+// RGGtkBuilderUserDialog
+RGGtkBuilderUserDialog::RGGtkBuilderUserDialog(RGWindow *parent, const char *name)
 {
    _parentWindow = parent->window();
    init(name);
 }
 
-bool RGGladeUserDialog::init(const char *name)
+bool RGGtkBuilderUserDialog::init(const char *name)
 {
    gchar *filename = NULL;
    gchar *main_widget = NULL;
+   guint builder_status;
+   GError* error = NULL;
 
-   //cout << "RGGladeUserDialog::RGGladeUserDialog()" << endl;
+   //cout << "RGGtkBuilderUserDialog::RGGtkBuilderUserDialog() " << name << endl;
 
-   filename = g_strdup_printf("dialog_%s.glade", name);
+   builder = gtk_builder_new();
+   filename = g_strdup_printf("gtkbuilder/dialog_%s.ui", name);
    main_widget = g_strdup_printf("dialog_%s", name);
    if (FileExists(filename)) {
-      gladeXML = glade_xml_new(filename, main_widget, NULL);
+      if (!gtk_builder_add_from_file (builder, filename, &error)) {
+         g_warning ("Couldn't load builder file: %s", error->message);
+         g_error_free (error);
+      }
    } else {
       g_free(filename);
-      filename = g_strdup_printf(SYNAPTIC_GLADEDIR "dialog_%s.glade", name);
-      gladeXML = glade_xml_new(filename, main_widget, NULL);
+      filename = g_strdup_printf(SYNAPTIC_GTKBUILDERDIR "dialog_%s.ui", name);
+      if (!gtk_builder_add_from_file (builder, filename, &error)) {
+         g_warning ("Couldn't load builder file: %s", error->message);
+         g_error_free (error);
+      }
    }
-   assert(gladeXML);
-   _dialog = glade_xml_get_widget(gladeXML, main_widget);
+   _dialog = GTK_WIDGET(gtk_builder_get_object(builder, main_widget));
    assert(_dialog);
 
    gtk_window_set_position(GTK_WINDOW(_dialog),
@@ -233,17 +247,19 @@ bool RGGladeUserDialog::init(const char *name)
 
    // honor foreign parent windows (to make embedding easy)
    int id = _config->FindI("Volatile::ParentWindowId", -1);
-   if (id > 0 && _parentWindow == NULL) {
+   if (id > 0 && !GTK_WIDGET_VISIBLE(_parentWindow)) {
       GdkWindow *win = gdk_window_foreign_new(id);
-      gtk_widget_realize(_dialog);
-      gdk_window_set_transient_for(GDK_WINDOW(_dialog->window), win);
+      if(win) {
+	 gtk_widget_realize(_dialog);
+	 gdk_window_set_transient_for(GDK_WINDOW(_dialog->window), win);
+      }
    }
 
    g_free(filename);
    g_free(main_widget);
 }
 
-int RGGladeUserDialog::run(const char *name,bool return_gtk_response)
+int RGGtkBuilderUserDialog::run(const char *name,bool return_gtk_response)
 {
    if(name != NULL)
       init(name);
